program scopeAutoArmSave
option +r;

/* Written by:
 *
 *	Nick DiMonte
 *	Argonne National Labratory
 *	April 07, 2003
 *
 *
 * Modifications:
 * ==============
 * .01	04-07-2003	npd	initial release
 * .02	06-16-2003	npd	Added SR current and User mode
 *                  to prevent unwanted triggers.
 * .03	06-19-2003	npd	Added a time stamp pv, when a
 *                  trigger is detected, this script 
 *                  will process the timestamp pv.
 * .04	01-19-2004	npd	Modified to work with 3014B network
 *                  scope.
 * .05	02-17-2004	npd	Selected network printer and format
 *                  type. Also added event status to check
 *                  printer communications. This to try
 *                  resolve the issue when a printer ggoes
 *                  into power save mode and takes longer to
 *                  power-up than the scope is willing to
 *                  wait.
 * .06	07-01-2011	npd	Create a version for the TDS5000
 *
 */

%%#include <stdio.h>	/* Standard I?O Functions */
%%#include <string.h>	/* String Library Functions */
%%#include <tsDefs.h>	/* Time Stamp Valuse */
%%#include <stdlib.h>	/* Library Functions */
%%#include <epicsThread.h>  /* New Library Functions replace taskLib.h */

#define REMOTE 0
#define LOCAL 1
#define DONE 0
#define OFF 0
#define ON 1
#define MEM1 0
#define SEQ 1
#define RUNSTOP 0
#define RUN 1
#define STOP 0
#define BUSY 1
#define ENABLE 1
#define DISABLE 0
#define NOUPDATE 1
#define UPDATE 0
#define USER 1

short RemoteCtl;
short RecallSetup;
short AutoPrintArm;
short ScopeBusy;
short ScopeReset;
short AcquireStat;
short AcquireStat_P;
short SingleSeqMode;
short SingleSeqStat;
short SingleSeqStat_P;
short AcquireRun;
short AcquireStop;
short PrintSetup;
short ProcessCtl;
short WF_ScanCtl;
short ScriptScanCtl;
short RunScanCtl;
string ScopeName;

assign RemoteCtl        to "{scope}GPIBenableBO";
assign RecallSetup      to "{scope}recallMO";
assign AutoPrintArm     to "{scope}armPrintAutoBO";
assign ScopeBusy        to "{scope}scopeBusyBI";
assign ScopeReset       to "{scope}resetBO";
assign AcquireStat      to "{scope}acquireStatBI";
assign AcquireStat_P    to "{scope}acquireStatBI.PROC";
assign SingleSeqMode    to "{scope}singleSeqBO";
assign SingleSeqStat    to "{scope}singleSeqBI";
assign SingleSeqStat_P  to "{scope}singleSeqBI.PROC";
assign AcquireRun       to "{scope}runBO";
assign AcquireStop      to "{scope}stopBO";
assign PrintSetup       to "{scope}printerSetupBO";
assign ProcessCtl       to "{scope}scopeInUseMO";
assign WF_ScanCtl       to "{scope}readScaledWfsCC.DISV";
assign ScriptScanCtl    to "{scope}ScriptUpdateCtlBO";
assign RunScanCtl       to "{scope}stopReadRunSQ.SCAN";
assign ScopeName        to "{scope}Name.DESC";

monitor RemoteCtl;
monitor ScopeBusy;
monitor AcquireStat;
monitor AutoPrintArm;
monitor SingleSeqStat;
monitor ScopeName;

short WriteFileCh1;
short WriteFileCh2;
short WriteFileCh3;
short WriteFileCh4;
assign WriteFileCh1  to "{scope}WrFileCh1WF.PROC";
assign WriteFileCh2  to "{scope}WrFileCh2WF.PROC";
assign WriteFileCh3  to "{scope}WrFileCh3WF.PROC";
assign WriteFileCh4  to "{scope}WrFileCh4WF.PROC";

unsigned int FileNameCh1;
unsigned int FileNameCh2;
unsigned int FileNameCh3;
unsigned int FileNameCh4;
assign FileNameCh1  to "{scope}WrFileCh1WF.NORD";
assign FileNameCh2  to "{scope}WrFileCh2WF.NORD";
assign FileNameCh3  to "{scope}WrFileCh3WF.NORD";
assign FileNameCh4  to "{scope}WrFileCh4WF.NORD";
monitor FileNameCh1;
monitor FileNameCh2;
monitor FileNameCh3;
monitor FileNameCh4;

short Ch1State;
short Ch2State;
short Ch3State;
short Ch4State;
assign Ch1State  to "{scope}chan1EnableBI";
assign Ch2State  to "{scope}chan1EnableBI";
assign Ch3State  to "{scope}chan1EnableBI";
assign Ch4State  to "{scope}chan1EnableBI";
monitor Ch1State;
monitor Ch2State;
monitor Ch3State;
monitor Ch4State;

short TimeStampPROC;
assign TimeStampPROC to "{scope}WF_timeStampTS.PROC";

float DESER_Ctl;
float SESR_Stat;
float EventQ;
short ClrEventQ;
short SaveError;
assign DESER_Ctl      to "{scope}DESERegAO";
assign SESR_Stat      to "{scope}SESRegAI";
assign EventQ         to "{scope}EventQueueAI";
assign ClrEventQ      to "{scope}ClearEventsBO";
assign SaveError     to "{scope}printErrBO";

short saveCnt;
short saveErr;
char name[8];

%%
ss ArmPrintTds5000 {
  state init {
    when( ) {
      printf("\n\nEntering Tds5000 Acquire & Save Seq !!!!\n");
      pvGet(ScopeName);
      strcpy(name,ScopeName); 
      printf("Configuring Tds5000 for %s \n",name);
      /* reset clears the file name to "untitled" */
      /* Filename could be stored in MEM1 where a RST could not clear. */
      /* ScopeReset = 1; */
      /* pvPut(ScopeReset); */
%%    epicsThreadSleep(5);       /* 5 second delay */
      ProcessCtl = 0;  /* Set to AVAILABLE */
      pvPut(ProcessCtl);
      ScriptScanCtl = UPDATE;
      pvPut(ScriptScanCtl);
      RecallSetup = MEM1;
      pvPut(RecallSetup);
%%    epicsThreadSleep(5); 
    } state remoteModeChk
  }
  state remoteModeChk {
    when ( RemoteCtl == REMOTE && AutoPrintArm == ON ) {
      /* WF_ScanCtl = DISABLE;
      pvPut(WF_ScanCtl); */
      RunScanCtl = DISABLE;
      pvPut(RunScanCtl); 
      printf("%s: Auto File Save set to ON !!!!\n",name);
    } state configScopeAuto
    when( RemoteCtl == LOCAL || AutoPrintArm == OFF ) {
      ProcessCtl = 0;  /* Set to AVAILABLE */
      pvPut(ProcessCtl);
      ScriptScanCtl = UPDATE;
      pvPut(ScriptScanCtl);
      /* WF_ScanCtl = ENABLE;
      pvPut(WF_ScanCtl); */
      printf("%s: Auto File Save set to OFF !!!!\n",name); 
    } state remoteModeWait
  }
  state remoteModeWait {
    when ( RemoteCtl == REMOTE && AutoPrintArm == ON ) {
      /* change detected..... */
    } state remoteModeChk
  }
  state configScopeAuto {
    when ( delay(3.0) ) {
      printf("%s: .... configuring scope, recall MEM1... \n",name); 
      ScriptScanCtl = NOUPDATE;
      pvPut(ScriptScanCtl);
%%    epicsThreadSleep(5); 
      RecallSetup = MEM1;
      pvPut(RecallSetup);
%%    epicsThreadSleep(5); 
      SingleSeqMode = RUNSTOP;
      pvPut(SingleSeqMode);
      ProcessCtl = 1;  /* Set to Auto Print mode */
      pvPut(ProcessCtl);
%%    epicsThreadSleep(5); 
      /* WF_ScanCtl = ENABLE;
      pvPut(WF_ScanCtl); */
    } state AcquireStop
  } 
  state AcquireStop {
    when ( delay(3.0) ) {
      printf("%s: .... set scope acquire to stop \n",name); 
      AcquireStop = 1;
      pvPut(AcquireStop);
      AcquireStat_P = 1;
      pvPut(AcquireStat_P);
%%    epicsThreadSleep(1);
    } state chkAcquireStat
  } 
  state chkAcquireStat {
    when ( RemoteCtl == REMOTE && AutoPrintArm == ON 
        && AcquireStat == STOP && ScopeBusy == DONE ) {
      printf("%s: Scope is NOT busy (good) ....\n",name); 
    } state armForSingleSeq
    when ( RemoteCtl == REMOTE && AutoPrintArm == ON 
        && AcquireStat == RUN && ScopeBusy == BUSY ) {
      printf("%s: Scope is STILL busy  acquiring ....\n",name); 
    } state AcquireStop
    when ( RemoteCtl == REMOTE && AutoPrintArm == ON 
        && AcquireStat == STOP && ScopeBusy == BUSY ) {
      printf("%s: Scope is STILL busy  printing ....\n",name); 
    } state configPrinterPort
    when ( RemoteCtl == REMOTE && AutoPrintArm == ON 
        && AcquireStat == RUN && ScopeBusy == DONE ) {
      printf("%s: Error: scope is in an invalid mode !!!!\n",name); 
      printf("       resetting scope !!!!\n"); 
      /* ScopeReset = 1;
      pvPut(ScopeReset); */
      SingleSeqMode = RUNSTOP;
      pvPut(SingleSeqMode);
      AcquireStop = 1;
      pvPut(AcquireStop);
    } state remoteModeChk
    when ( RemoteCtl == LOCAL || AutoPrintArm == OFF ) {
      printf("%s: .... chkAcquireStat: auto print canceled \n",name); 
      ProcessCtl = 0;  /* Set to AVAILABLE */
      pvPut(ProcessCtl);
    } state remoteModeChk
  } 
  state armForSingleSeq {
    when ( delay(3.4) ) {
      printf("%s: .... arming scope for single seq \n",name); 
      SingleSeqMode = SEQ;
      pvPut(SingleSeqMode);
      AcquireRun = RUN;
      pvPut(AcquireRun);
%%    epicsThreadSleep(1);
      SingleSeqStat_P = 1;
      pvPut(SingleSeqStat_P);
%%    epicsThreadSleep(3);
    } state chkSglSeq
  } 
  state chkSglSeq {
    when ( SingleSeqStat == SEQ) {
    } state waitForSglSeq
    when ( SingleSeqStat == RUNSTOP) {
      printf("%s: ERROR: scope not set for sequence mode \n",name);
    } state remoteModeChk
  } 
  state waitForSglSeq {
    when ( RemoteCtl == REMOTE && AutoPrintArm == ON 
        && AcquireStat == RUN && ScopeBusy == BUSY && SingleSeqStat == SEQ ) {
      printf("%s: .... waiting for scope to be armed \n",name); 
    } state waitForTriggerDly
    when ( RemoteCtl == LOCAL || AutoPrintArm == OFF ||
                                          SingleSeqStat == RUNSTOP ) {
      printf("%s: .... waitForSglSeq: auto print canceled \n",name); 
      ProcessCtl = 0;  /* Set to AVAILABLE */
      pvPut(ProcessCtl);
    } state remoteModeChk
  } 
  state waitForTriggerDly {
    when ( delay(3.0) ) {
      printf("%s: .... waiting for scope to trigger \n",name); 
    } state waitForTrigger
  }
  state waitForTrigger {
    when ( RemoteCtl == REMOTE && AutoPrintArm == ON  
        && AcquireStat == DONE && ScopeBusy == DONE && SingleSeqStat == SEQ ) {
      printf("%s: .... trigger has been received \n",name); 
      ScriptScanCtl = UPDATE;
      pvPut(ScriptScanCtl);
      TimeStampPROC = 1;
      pvPut(TimeStampPROC);
      saveCnt=0;
      SaveError = 0; /* OK */
      pvPut(SaveError);
    } state saveFilesDly
    when ( RemoteCtl == LOCAL || AutoPrintArm == OFF ||
                                    SingleSeqStat == RUNSTOP ) {
      printf("%s: .... waitForTrigger: auto print canceled \n",name); 
      printf("     RemoteCtl     = %d\n",RemoteCtl);
      printf("     AutoPrintArm  = %d\n",AutoPrintArm);
      printf("     AcquireStat   = %d\n",AcquireStat);
      printf("     ScopeBusy     = %d\n",ScopeBusy);
      printf("     SingleSeqStat = %d\n",SingleSeqStat);
      ProcessCtl = 0;  /* Set to AVAILABLE */
      pvPut(ProcessCtl);
    } state remoteModeChk
  } 
  state saveFilesDly {
    when ( delay(10.0) ) {
      /* WF_ScanCtl = DISABLE;
      pvPut(WF_ScanCtl); */
      DESER_Ctl = 0;
      pvPut(DESER_Ctl); 
%%    epicsThreadSleep(5);       /* 5 second delay */
      ClrEventQ = 1;
      pvPut(ClrEventQ); 
%%    epicsThreadSleep(5);       /* 5 second delay */
      DESER_Ctl = 16;
      pvPut(DESER_Ctl); 
    } state saveFiles
  }
  state saveFiles {
    when ( delay(2.0) ) {
      pvGet(FileNameCh1);
      printf("Ch1State = %d, NORD = %d\n",Ch1State,FileNameCh1);
      if ( Ch1State == ON && FileNameCh1 > 5 ){
        WriteFileCh1 = 1;
        pvPut(WriteFileCh1);
        printf("Saving ch1 to file...\n");
      }
      pvGet(FileNameCh2);
      printf("Ch2State = %d, NORD = %d\n",Ch2State,FileNameCh2);
      if ( Ch2State == ON && FileNameCh2 > 5 ){
        WriteFileCh2 = 1;
        pvPut(WriteFileCh2);
        printf("Saving ch2 to file...\n");
      }
      pvGet(FileNameCh3);
      printf("Ch3State = %d, NORD = %d\n",Ch3State,FileNameCh3);
      if ( Ch3State == ON && FileNameCh3 > 5 ){
        WriteFileCh3 = 1;
        pvPut(WriteFileCh3);
        printf("Saving ch3 to file...\n");
      }
      pvGet(FileNameCh4);
      printf("Ch4State = %d, NORD = %d\n",Ch4State,FileNameCh4);
      if ( Ch4State == ON && FileNameCh4 > 5 ){
        WriteFileCh4 = 1;
        pvPut(WriteFileCh4);
        printf("Saving ch4 to file...\n");
      }
      saveCnt++;
      printf("%s: Saving Tds5000 waveforms to files !!!!\n",name);
    } state waitForFilesDoneDly
  } 
  state waitForFilesDoneDly {
    when ( delay(10.0) ) {
      printf("%s: .... waiting for save files to finish \n",name); 
    } state waitForFilesDone
  } 
  state waitForFilesDone {
    when ( RemoteCtl == REMOTE && AutoPrintArm == ON && ScopeBusy == DONE ) {
      saveErr=0;
      while ( (pvGet(SESR_Stat) & 0x10) > 0 ){
        pvGet(EventQ);
        printf("%s: Event error #= %f\n",name,EventQ);
        if ( EventQ >= 2209 && EventQ <= 2420 ){
          saveErr++;
        }	
      }
    } state chkSaveErrors
    when ( RemoteCtl == LOCAL || AutoPrintArm == OFF ) {
    } state configScopeAuto
  } 
  state chkSaveErrors {
    when ( saveErr > 0 && saveCnt < 6 ) {
%%    epicsThreadSleep(60);       /* 60 second delay */
      printf("%s: .... error saving files, attempting again \n",name); 
    } state saveFiles
    when ( saveErr == 0 ) {
      printf("%s: .... save files was successful. \n",name); 
      SaveError = 0; /* OK */
      pvPut(SaveError);
    } state saveFilesDone
    when ( saveCnt == 6 ) {
      printf("%s: .... save files failed, aborting.... \n",name); 
      SaveError = 1; /* Fault */
      pvPut(SaveError);
    } state saveFilesDone
  } 
  state saveFilesDone {
    when ( RemoteCtl == REMOTE && AutoPrintArm == ON && ScopeBusy == DONE ) {
      ScriptScanCtl = NOUPDATE;
      pvPut(ScriptScanCtl);
      /* WF_ScanCtl = ENABLE;
      pvPut(WF_ScanCtl); */
      printf("%s: Save files for Tds5000 DONE !!!!\n",name);
    } state configScopeAuto
    when ( RemoteCtl == LOCAL || AutoPrintArm == OFF ) {
      printf("%s: .... waitForSaveFile: auto save canceled \n",name); 
      ScriptScanCtl = UPDATE;
      pvPut(ScriptScanCtl);
      /* WF_ScanCtl = ENABLE;
      pvPut(WF_ScanCtl); */
      ProcessCtl = 0;  /* Set to AVAILABLE */
      pvPut(ProcessCtl);
    } state remoteModeChk
  } 
}

